/**
 * An abstract class to extend and create global events.
 *
 * @class Event
 */
AJS.Event = Class.extend({

    /**
     * 
     * @constructor
     * @param options
     */
    init: function (options) {
        this.$target = AJS.$(options.target);
        this.eventOwner = options.eventOwner;
    },

    /**
     * Creates interface for event. This object is passed to event handlers.
     *
     * @method getEventInterface
     * @return {Object}
     * ... {jQuery} target - DOM element that relates to event. This is the element the event will be fired on.
     * ... {Class} eventOwner - Class that relates to event
     */
    getEventInterface: function() {

        var instance = this;

        return {

            target: function () {
                return instance.$target;
            },

            eventOwner: function () {
                return instance.eventOwner;
            }
            
        };
    },

    /**
     * Fires event on DOM element defined in the options [target] property
     *
     * @method fire
     * @return * - This is the result of the return statement of event handlers.
     */
    fire: function () {
        var event = new AJS.$.Event(this.EVENT_NAME);
        AJS.$(this.$target).trigger(event, [this.getEventInterface()]);
        return event.result;
    }

});


/**
 * A listener to content changes. The [target] property of eventInterface is the parent object of the DOM elements
 * whose content has changed. You can use this event to trigger and loosly couple event handlers when building dynamic
 * elements in javascript.
 *
 * <h5>Usage</h5>
 *
 * <pre>
 *
 * // fire event
 * var event = new AJS.ContentChangeEvent({
 *      target: AJS.$("#myparentelementofwhomscontenthaschanged"),
 *      eventOwner: AJS.InlineLayer
 * });
 *
 * event.fire();
 *
 * </pre>
 *
 * @event contentChangeEvent
 * @param {e} Event
 * @param {Object} eventInterface
 * @namespace AJS
 */
AJS.ContentChangeEvent = AJS.Event.extend({
    EVENT_NAME: "contentChange"
});