/**
 * Inline dialogs for user infomation:
 *
 * @addon jira.app.userhover
 *   -- To reduce unneeded HTTP requests (JRADEV-2207) we need to circumvent AJS.InlineDialog's default
 *      behaviour. Thus, we use the noBind = true option, and manually control when to show/hide the
 *      dialog popups.
 *
 * @param {HTMLElement} context (optional)
 *   -- The scope in which to look for "a.user-hover" elements
 */
jira.app.userhover = function(context) {
    AJS.$("a.user-hover", context).bind({
        "mouseenter": function() {
            var trigger = this,
                options = jira.app.userhover.INLINE_DIALOG_OPTIONS,
                control = AJS.$.data(trigger, "AJS.InlineDialog") || AJS.$.data(trigger, "AJS.InlineDialog", {});

            control.hasUserAttention = true;
            clearTimeout(control.timerId);
            control.timerId = setTimeout(function() {
                if (control.show) {
                    control.show();
                } else {
                    var $trigger = AJS.$(trigger),
                        $popup = AJS.InlineDialog(
                            $trigger,
                            "user-hover-dialog-" + new Date().getTime(),
                            function($contents, control, show) {
                                // Call the InlineDialog's url function with its expected arguments.
                                jira.app.userhover.fetchDialogContents($contents, $trigger, show);
                            },
                            options);

                    $popup.show();
                    $popup.find(".contents").bind({
                        "mouseenter": function() {
                            control.hasUserAttention = true;
                        },
                        "mouseleave": function() {
                            $trigger.mouseleave();
                        }
                    });

                    control.show = $popup.show;
                    control.hide = $popup.hide;
                }
            }, options.showDelay);
        },
        "mouseleave": function() {
            var control = AJS.$.data(this, "AJS.InlineDialog");

            control.hasUserAttention = false;
            clearTimeout(control.timerId);

            if (control.hide) {
                control.timerId = setTimeout(function() {
                    if (!control.hasUserAttention) {
                        // Close the 'More' submenu.
                        if (AJS.dropDown.current) {
                            AJS.dropDown.current.hide();
                        }
                        control.hide();
                    }
                }, jira.app.userhover.INLINE_DIALOG_OPTIONS.showDelay);
            }
        }
    });
};

jira.app.userhover.INLINE_DIALOG_OPTIONS = {
    urlPrefix: contextPath + "/secure/ViewUserHover!default.jspa?decorator=none&username=",
    showDelay: 400,
    closeOthers: false,
    noBind: true, // Using the noBind option, we must manually call the InlineDialog's show/hide methods at the appropriate times.
    hideCallback: function() {
        // Only hide the current dropdown if it's internal to an inline dialog.
        if (AJS.dropDown.current && AJS.$(AJS.dropDown.current.trigger).parents(".aui-inline-dialog").length > 0) {
            AJS.dropDown.current.hide();
        }
    }
};

jira.app.userhover.fetchDialogContents = function($contents, $trigger, firstShow) {
    var trigger = $trigger[0],
        options = jira.app.userhover.INLINE_DIALOG_OPTIONS,
        control = AJS.$.data(trigger, "AJS.InlineDialog");

    if (!control.hasContent) {
        control.hasContent = true;

        AJS.$.get(options.urlPrefix + $trigger.attr("rel"), function(html) {
            if (control.hasUserAttention) {
                firstShow();
            } else {
                // Without calling firstShow, control.show won't have any effect, so we have to do some fancy footwork here.
                var show = control.show;
                control.show = function() {
                    firstShow();
                    control.show = show;
                };
            }
            $contents.html(html);
            $contents.css("overflow", "visible");
            $contents.find(".aui-dd-parent").dropDown("Standard", { trigger: ".aui-dd-link" });
            $contents.bind("click", options.hideCallback);
        });
    }
};

AJS.$(function() {
    jira.app.userhover(document.body);
});
